/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2021/06/06
* Description        : Main program body.
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* SPDX-License-Identifier: Apache-2.0
*******************************************************************************/

/*
 *@Note
 task1 and task2 alternate printing
*/

#include "debug.h"
#include "Uart.h"

#include "Timer.h"

#include "FreeRTOS.h"
#include "task.h"

#include "Jubei_eth_driver.h"
#include "lwip/ip4_addr.h"

/* Global define */
#define TASK1_TASK_PRIO     5
#define TASK1_STK_SIZE      256
#define TASK2_TASK_PRIO     5
#define TASK2_STK_SIZE      1256

/* Global Variable */
TaskHandle_t Task1Task_Handler;
TaskHandle_t Task2Task_Handler;


/*********************************************************************
 * @fn      GPIO_Toggle_INIT
 *
 * @brief   Initializes GPIOA.0/1
 *
 * @return  none
 */
void GPIO_Toggle_INIT(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure={0};

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA,ENABLE);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}


/*********************************************************************
 * @fn      task1_task
 *
 * @brief   task1 program.
 *
 * @param  *pvParameters - Parameters point of task1
 *
 * @return  none
 */
void Uart_task(void *pvParameters)
{

    Uart2Init();
    Uart4Init();
    Uart5Init();

    InitBaseTimer6();

    while(1)
    {
        printf("t1 entry\r\n");
        GPIO_SetBits(GPIOA, GPIO_Pin_0);
        vTaskDelay(250);
        GPIO_ResetBits(GPIOA, GPIO_Pin_0);
        vTaskDelay(250);


        taskENTER_CRITICAL();
        if(UartRxdCtrl[0].RxdOk==0xaa)
        {
            printf("Uart2 GetData %d bytes\r\n",UartRxdCtrl[0].RxdLen);
            Uart2SendRaw(UartRxdCtrl[0].RxdBuf,UartRxdCtrl[0].RxdLen);
            UartRxdCtrl[0].RxdOk=0;
        }
        taskEXIT_CRITICAL();

        taskENTER_CRITICAL();
        if(UartRxdCtrl[1].RxdOk==0xaa)
        {
            printf("Uart4 GetData\r\n");
            Uart4SendRaw(UartRxdCtrl[1].RxdBuf,UartRxdCtrl[1].RxdLen);
            UartRxdCtrl[1].RxdOk=0;
        }
        taskEXIT_CRITICAL();

        taskENTER_CRITICAL();
        if(UartRxdCtrl[2].RxdOk==0xaa)
        {
            printf("Uart5 GetData\r\n");
            Uart5SendRaw(UartRxdCtrl[2].RxdBuf,UartRxdCtrl[2].RxdLen);
            UartRxdCtrl[2].RxdOk=0;
        }
        taskEXIT_CRITICAL();

    }
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
static void echo_client_init(void);

//ʧܺ echo_client_init½ͻˣ¿ʼ
static void echo_client_conn_err(void *arg,err_t err)
{
    printf("connect error! closed by core!!\n");
    printf("try to connect to sever again!!\n");
    echo_client_init();
}

//յݺں˵ãpа˷
static err_t echo_client_recv(void *arg,struct tcp_pcb *pcb,struct pbuf *p,err_t err)
{
    if(p!=NULL)
    {
        tcp_recved(pcb, p->tot_len);     //½մ
        tcp_write(pcb, p->payload, p->len, 1);   //Է
        pbuf_free(p);
    }else if(err == ERR_OK)   //Ͽ
    {
        tcp_close(pcb);      //ȶϿ
        echo_client_init();  //ٴνµ
        return ERR_OK;
    }
    return ERR_OK;
}

//ͻݳɹں˵
static err_t echo_client_sent(void *arg,struct tcp_pcb *pcb,uint16_t len)
{
    printf("echo client send data OK! sent len = %d\n",len);
    return ERR_OK;
}

//ͻͬӳɹں˵
static err_t echo_client_connected(void *arg,struct tcp_pcb *pcb,err_t err)
{
    char GREENTING[] = "Hi,I am a new Client!\n";

    tcp_recv(pcb,echo_client_recv);        //עrecv
    tcp_sent(pcb,echo_client_sent);        //עsent
    tcp_write(pcb,GREENTING,sizeof(GREENTING),1);     //GREETING
    return ERR_OK;
}

//ͻ˳ʼҪں˳ʼһ
static void echo_client_init(void)
{
    struct tcp_pcb *pcb = NULL;
    ip_addr_t server_ip;
    pcb = tcp_new();
    IP4_ADDR(&server_ip,192,168,1,11);
    tcp_bind(pcb, IP_ADDR_ANY, 20);
    tcp_connect(pcb,&server_ip,5,echo_client_connected);  //ӷ21˿
    tcp_err(pcb,echo_client_conn_err);                    //עӴʱĻص
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*********************************************************************
 * @fn      task2_task
 *
 * @brief   task2 program.
 *
 * @param  *pvParameters - Parameters point of task2
 *
 * @return  none
 */
void task2_task(void *pvParameters)
{

    LwIP_Init();
    echo_client_init();


    while(1)
    {
        printf("t2\r\n");
//        GPIO_ResetBits(GPIOA, GPIO_Pin_1);
//        vTaskDelay(500);
//        GPIO_SetBits(GPIOA, GPIO_Pin_1);
//        vTaskDelay(500);

        sys_check_timeouts();
        vTaskDelay(1);


    }
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	Delay_Init();
	USART_Printf_Init(115200);
	printf("SystemClk:%d\r\n",SystemCoreClock);
	printf("FreeRTOS Kernel Version:%s\r\n",tskKERNEL_VERSION_NUMBER);

	GPIO_Toggle_INIT();
	/* create two task */
    xTaskCreate((TaskFunction_t )task2_task,
                        (const char*    )"task2",
                        (uint16_t       )TASK2_STK_SIZE,
                        (void*          )NULL,
                        (UBaseType_t    )TASK2_TASK_PRIO,
                        (TaskHandle_t*  )&Task2Task_Handler);

    xTaskCreate((TaskFunction_t )Uart_task,
                    (const char*    )"task1",
                    (uint16_t       )TASK1_STK_SIZE,
                    (void*          )NULL,
                    (UBaseType_t    )TASK1_TASK_PRIO,
                    (TaskHandle_t*  )&Task1Task_Handler);
    vTaskStartScheduler();

	while(1)
	{
	    printf("shouldn't run at here!!\n");
	}
}
